searchData={"items":[{"type":"module","title":"Supertester","doc":"Supertester - Multi-repository test orchestration and execution framework.\n\nProvides centralized test management, execution, and reporting across\nmultiple Elixir repositories in a monorepo structure.","ref":"Supertester.html"},{"type":"module","title":"Core Modules - Supertester","doc":"- `Supertester.UnifiedTestFoundation` - Test isolation and foundation patterns\n- `Supertester.OTPHelpers` - OTP-compliant testing utilities\n- `Supertester.GenServerHelpers` - GenServer-specific test patterns\n- `Supertester.SupervisorHelpers` - Supervision tree testing utilities\n- `Supertester.MessageHelpers` - Message tracing and ETS management\n- `Supertester.PerformanceHelpers` - Benchmarking and load testing\n- `Supertester.ChaosHelpers` - Chaos engineering and resilience testing\n- `Supertester.DataGenerators` - Test data and scenario generation\n- `Supertester.Assertions` - Custom OTP-aware assertions","ref":"Supertester.html#module-core-modules"},{"type":"module","title":"Usage - Supertester","doc":"Add supertester as a test dependency in your mix.exs:\n\n    def deps do\n      [\n        {:supertester, path: \"../supertester\", only: :test}\n      ]\n    end\n\nThen use the helpers in your tests:\n\n    defmodule MyApp.MyModuleTest do\n      use ExUnit.Case, async: true\n      import Supertester.OTPHelpers\n      import Supertester.Assertions\n\n      describe \"my functionality\" do\n        setup do\n          setup_isolated_genserver(MyModule, \"my_test\")\n        end\n\n        test \"my test\", %{server: server} do\n          assert_genserver_responsive(server)\n        end\n      end\n    end","ref":"Supertester.html#module-usage"},{"type":"function","title":"Supertester.version/0","doc":"Returns the version of Supertester.","ref":"Supertester.html#version/0"},{"type":"module","title":"Supertester.UnifiedTestFoundation","doc":"Unified test foundation providing multiple isolation modes for OTP testing.\n\nThis module provides macros and utilities to establish different levels of test isolation,\nenabling async testing while preventing process conflicts and ensuring proper cleanup.","ref":"Supertester.UnifiedTestFoundation.html"},{"type":"module","title":"Isolation Modes - Supertester.UnifiedTestFoundation","doc":"- `:basic` - Basic isolation with unique naming\n- `:registry` - Registry-based process isolation\n- `:full_isolation` - Complete process and ETS isolation\n- `:contamination_detection` - Isolation with contamination detection","ref":"Supertester.UnifiedTestFoundation.html#module-isolation-modes"},{"type":"module","title":"Usage - Supertester.UnifiedTestFoundation","doc":"defmodule MyApp.MyModuleTest do\n      use ExUnit.Case\n      use Supertester.UnifiedTestFoundation, isolation: :full_isolation\n\n      test \"my test\", context do\n        # Test runs with full isolation\n      end\n    end","ref":"Supertester.UnifiedTestFoundation.html#module-usage"},{"type":"macro","title":"Supertester.UnifiedTestFoundation.__using__/1","doc":"Provides test isolation setup based on the specified isolation type.","ref":"Supertester.UnifiedTestFoundation.html#__using__/1"},{"type":"function","title":"Supertester.UnifiedTestFoundation.isolation_allows_async?/1","doc":"Returns whether the isolation type allows async testing.","ref":"Supertester.UnifiedTestFoundation.html#isolation_allows_async?/1"},{"type":"function","title":"Supertester.UnifiedTestFoundation.isolation_timeout/1","doc":"Returns the timeout for the isolation type.","ref":"Supertester.UnifiedTestFoundation.html#isolation_timeout/1"},{"type":"function","title":"Supertester.UnifiedTestFoundation.setup_isolation/2","doc":"Sets up test isolation based on the specified isolation type.","ref":"Supertester.UnifiedTestFoundation.html#setup_isolation/2"},{"type":"function","title":"Supertester.UnifiedTestFoundation.verify_test_isolation/1","doc":"Verifies test isolation has been maintained.","ref":"Supertester.UnifiedTestFoundation.html#verify_test_isolation/1"},{"type":"function","title":"Supertester.UnifiedTestFoundation.wait_for_supervision_tree_ready/2","doc":"Waits for a supervision tree to be ready.","ref":"Supertester.UnifiedTestFoundation.html#wait_for_supervision_tree_ready/2"},{"type":"module","title":"Supertester.GenServerHelpers","doc":"Specialized helpers for GenServer testing patterns.\n\nThis module provides utilities specifically designed for testing GenServer behavior,\nincluding state management, concurrent testing, and error scenarios.","ref":"Supertester.GenServerHelpers.html"},{"type":"module","title":"Key Features - Supertester.GenServerHelpers","doc":"- Safe GenServer state access\n- Cast and sync patterns for proper async operation testing\n- Concurrent testing utilities\n- Error testing and crash recovery\n- Timeout-aware GenServer operations","ref":"Supertester.GenServerHelpers.html#module-key-features"},{"type":"module","title":"Usage - Supertester.GenServerHelpers","doc":"import Supertester.GenServerHelpers\n\n    test \"genserver state management\" do\n      {:ok, server} = setup_isolated_genserver(MyGenServer)\n      state = get_server_state_safely(server)\n      assert state.counter == 0\n    end","ref":"Supertester.GenServerHelpers.html#module-usage"},{"type":"function","title":"Supertester.GenServerHelpers.call_with_timeout/3","doc":"Makes a GenServer call with timeout handling.","ref":"Supertester.GenServerHelpers.html#call_with_timeout/3"},{"type":"function","title":"Parameters - Supertester.GenServerHelpers.call_with_timeout/3","doc":"- `server` - The GenServer pid or name\n- `message` - The message to send\n- `timeout` - Timeout in milliseconds (default: 1000)","ref":"Supertester.GenServerHelpers.html#call_with_timeout/3-parameters"},{"type":"function","title":"Returns - Supertester.GenServerHelpers.call_with_timeout/3","doc":"`{:ok, response}` if successful, `{:error, reason}` otherwise","ref":"Supertester.GenServerHelpers.html#call_with_timeout/3-returns"},{"type":"function","title":"Example - Supertester.GenServerHelpers.call_with_timeout/3","doc":"{:ok, response} = call_with_timeout(server, :get_counter, 5000)","ref":"Supertester.GenServerHelpers.html#call_with_timeout/3-example"},{"type":"function","title":"Supertester.GenServerHelpers.cast_and_sync/3","doc":"Sends a cast and then synchronizes with a sync message.\n\nThis is crucial for testing async operations - it ensures the cast has been\nprocessed before the test continues.","ref":"Supertester.GenServerHelpers.html#cast_and_sync/3"},{"type":"function","title":"Parameters - Supertester.GenServerHelpers.cast_and_sync/3","doc":"- `server` - The GenServer pid or name\n- `cast_message` - The async message to cast\n- `sync_message` - The sync message for synchronization (default: :__supertester_sync__)","ref":"Supertester.GenServerHelpers.html#cast_and_sync/3-parameters"},{"type":"function","title":"Returns - Supertester.GenServerHelpers.cast_and_sync/3","doc":"`:ok` if successful, `{:error, reason}` otherwise","ref":"Supertester.GenServerHelpers.html#cast_and_sync/3-returns"},{"type":"function","title":"Example - Supertester.GenServerHelpers.cast_and_sync/3","doc":":ok = cast_and_sync(server, {:increment, 5})\n    {:ok, state} = get_server_state_safely(server)\n    assert state.counter == 5","ref":"Supertester.GenServerHelpers.html#cast_and_sync/3-example"},{"type":"function","title":"Supertester.GenServerHelpers.concurrent_calls/3","doc":"Performs concurrent calls to a GenServer for stress testing.","ref":"Supertester.GenServerHelpers.html#concurrent_calls/3"},{"type":"function","title":"Parameters - Supertester.GenServerHelpers.concurrent_calls/3","doc":"- `server` - The GenServer pid or name\n- `calls` - List of messages to send concurrently\n- `count` - Number of concurrent processes per message (default: 10)","ref":"Supertester.GenServerHelpers.html#concurrent_calls/3-parameters"},{"type":"function","title":"Returns - Supertester.GenServerHelpers.concurrent_calls/3","doc":"`{:ok, results}` with list of {call, result} tuples","ref":"Supertester.GenServerHelpers.html#concurrent_calls/3-returns"},{"type":"function","title":"Example - Supertester.GenServerHelpers.concurrent_calls/3","doc":"calls = [:get_counter, {:increment, 1}, :get_counter]\n    {:ok, results} = concurrent_calls(server, calls, 5)","ref":"Supertester.GenServerHelpers.html#concurrent_calls/3-example"},{"type":"function","title":"Supertester.GenServerHelpers.get_server_state_safely/1","doc":"Safely retrieves the internal state of a GenServer.\n\nThis function attempts to get the GenServer state without causing crashes\nif the server is not responsive or doesn't support state inspection.","ref":"Supertester.GenServerHelpers.html#get_server_state_safely/1"},{"type":"function","title":"Parameters - Supertester.GenServerHelpers.get_server_state_safely/1","doc":"- `server` - The GenServer pid or name","ref":"Supertester.GenServerHelpers.html#get_server_state_safely/1-parameters"},{"type":"function","title":"Returns - Supertester.GenServerHelpers.get_server_state_safely/1","doc":"`{:ok, state}` if successful, `{:error, reason}` otherwise","ref":"Supertester.GenServerHelpers.html#get_server_state_safely/1-returns"},{"type":"function","title":"Example - Supertester.GenServerHelpers.get_server_state_safely/1","doc":"{:ok, state} = get_server_state_safely(my_server)","ref":"Supertester.GenServerHelpers.html#get_server_state_safely/1-example"},{"type":"function","title":"Supertester.GenServerHelpers.stress_test_server/3","doc":"Performs stress testing on a GenServer with various operations.","ref":"Supertester.GenServerHelpers.html#stress_test_server/3"},{"type":"function","title":"Parameters - Supertester.GenServerHelpers.stress_test_server/3","doc":"- `server` - The GenServer pid or name\n- `operations` - List of operations to perform randomly\n- `duration` - Duration in milliseconds (default: 5000)","ref":"Supertester.GenServerHelpers.html#stress_test_server/3-parameters"},{"type":"function","title":"Returns - Supertester.GenServerHelpers.stress_test_server/3","doc":"`{:ok, stats}` with stress test statistics","ref":"Supertester.GenServerHelpers.html#stress_test_server/3-returns"},{"type":"function","title":"Example - Supertester.GenServerHelpers.stress_test_server/3","doc":"operations = [\n      {:call, :get_counter},\n      {:cast, {:increment, 1}},\n      {:call, {:add, 5}}\n    ]\n    {:ok, stats} = stress_test_server(server, operations, 10_000)","ref":"Supertester.GenServerHelpers.html#stress_test_server/3-example"},{"type":"function","title":"Supertester.GenServerHelpers.test_invalid_messages/2","doc":"Tests how a GenServer handles invalid messages.","ref":"Supertester.GenServerHelpers.html#test_invalid_messages/2"},{"type":"function","title":"Parameters - Supertester.GenServerHelpers.test_invalid_messages/2","doc":"- `server` - The GenServer pid or name\n- `invalid_messages` - List of invalid messages to test","ref":"Supertester.GenServerHelpers.html#test_invalid_messages/2-parameters"},{"type":"function","title":"Returns - Supertester.GenServerHelpers.test_invalid_messages/2","doc":"`{:ok, results}` with test results for each message","ref":"Supertester.GenServerHelpers.html#test_invalid_messages/2-returns"},{"type":"function","title":"Example - Supertester.GenServerHelpers.test_invalid_messages/2","doc":"invalid_messages = [:invalid_call, {:unknown, :message}, \"string_message\"]\n    {:ok, results} = test_invalid_messages(server, invalid_messages)","ref":"Supertester.GenServerHelpers.html#test_invalid_messages/2-example"},{"type":"function","title":"Supertester.GenServerHelpers.test_server_crash_recovery/2","doc":"Tests GenServer crash recovery behavior.","ref":"Supertester.GenServerHelpers.html#test_server_crash_recovery/2"},{"type":"function","title":"Parameters - Supertester.GenServerHelpers.test_server_crash_recovery/2","doc":"- `server` - The GenServer pid or name\n- `crash_reason` - The reason to crash the server with","ref":"Supertester.GenServerHelpers.html#test_server_crash_recovery/2-parameters"},{"type":"function","title":"Returns - Supertester.GenServerHelpers.test_server_crash_recovery/2","doc":"`{:ok, recovery_info}` if recovery successful, `{:error, reason}` otherwise","ref":"Supertester.GenServerHelpers.html#test_server_crash_recovery/2-returns"},{"type":"function","title":"Example - Supertester.GenServerHelpers.test_server_crash_recovery/2","doc":"{:ok, info} = test_server_crash_recovery(server, :test_crash)\n    assert info.recovered == true","ref":"Supertester.GenServerHelpers.html#test_server_crash_recovery/2-example"},{"type":"module","title":"Supertester.OTPHelpers","doc":"OTP-compliant testing utilities for GenServer, Supervisor, and process management.\n\nThis module provides helpers that replace timing-based synchronization (Process.sleep)\nwith proper OTP synchronization patterns, enabling reliable async testing.","ref":"Supertester.OTPHelpers.html"},{"type":"module","title":"Key Features - Supertester.OTPHelpers","doc":"- Unique process naming to prevent conflicts\n- OTP-compliant synchronization without Process.sleep\n- Automatic resource cleanup\n- Process lifecycle management\n- Supervision tree testing utilities","ref":"Supertester.OTPHelpers.html#module-key-features"},{"type":"module","title":"Usage - Supertester.OTPHelpers","doc":"import Supertester.OTPHelpers\n\n    test \"my genserver test\" do\n      {:ok, server} = setup_isolated_genserver(MyGenServer, \"test_context\")\n      assert_genserver_responsive(server)\n    end","ref":"Supertester.OTPHelpers.html#module-usage"},{"type":"function","title":"Supertester.OTPHelpers.cleanup_on_exit/1","doc":"Registers a cleanup function to run on test exit.","ref":"Supertester.OTPHelpers.html#cleanup_on_exit/1"},{"type":"function","title":"Parameters - Supertester.OTPHelpers.cleanup_on_exit/1","doc":"- `cleanup_fun` - Function to call on test exit","ref":"Supertester.OTPHelpers.html#cleanup_on_exit/1-parameters"},{"type":"function","title":"Example - Supertester.OTPHelpers.cleanup_on_exit/1","doc":"cleanup_on_exit(fn -> GenServer.stop(my_server) end)","ref":"Supertester.OTPHelpers.html#cleanup_on_exit/1-example"},{"type":"function","title":"Supertester.OTPHelpers.cleanup_processes/1","doc":"Cleans up a list of processes safely.","ref":"Supertester.OTPHelpers.html#cleanup_processes/1"},{"type":"function","title":"Parameters - Supertester.OTPHelpers.cleanup_processes/1","doc":"- `pids` - List of PIDs to clean up","ref":"Supertester.OTPHelpers.html#cleanup_processes/1-parameters"},{"type":"function","title":"Example - Supertester.OTPHelpers.cleanup_processes/1","doc":"cleanup_processes([pid1, pid2, pid3])","ref":"Supertester.OTPHelpers.html#cleanup_processes/1-example"},{"type":"function","title":"Supertester.OTPHelpers.monitor_process_lifecycle/1","doc":"Monitors a process lifecycle and returns monitoring information.","ref":"Supertester.OTPHelpers.html#monitor_process_lifecycle/1"},{"type":"function","title":"Parameters - Supertester.OTPHelpers.monitor_process_lifecycle/1","doc":"- `pid` - The process to monitor","ref":"Supertester.OTPHelpers.html#monitor_process_lifecycle/1-parameters"},{"type":"function","title":"Returns - Supertester.OTPHelpers.monitor_process_lifecycle/1","doc":"`{monitor_ref, pid}`","ref":"Supertester.OTPHelpers.html#monitor_process_lifecycle/1-returns"},{"type":"function","title":"Example - Supertester.OTPHelpers.monitor_process_lifecycle/1","doc":"{ref, pid} = monitor_process_lifecycle(server_pid)","ref":"Supertester.OTPHelpers.html#monitor_process_lifecycle/1-example"},{"type":"function","title":"Supertester.OTPHelpers.setup_isolated_genserver/3","doc":"Sets up an isolated GenServer with unique naming and automatic cleanup.","ref":"Supertester.OTPHelpers.html#setup_isolated_genserver/3"},{"type":"function","title":"Parameters - Supertester.OTPHelpers.setup_isolated_genserver/3","doc":"- `module` - The GenServer module to start\n- `test_name` - Test context for unique naming (optional)\n- `opts` - Options passed to GenServer.start_link (optional)","ref":"Supertester.OTPHelpers.html#setup_isolated_genserver/3-parameters"},{"type":"function","title":"Returns - Supertester.OTPHelpers.setup_isolated_genserver/3","doc":"`{:ok, server_pid}` or `{:error, reason}`","ref":"Supertester.OTPHelpers.html#setup_isolated_genserver/3-returns"},{"type":"function","title":"Example - Supertester.OTPHelpers.setup_isolated_genserver/3","doc":"{:ok, server} = setup_isolated_genserver(MyGenServer, \"my_test\", [initial_state: %{}])","ref":"Supertester.OTPHelpers.html#setup_isolated_genserver/3-example"},{"type":"function","title":"Supertester.OTPHelpers.setup_isolated_supervisor/3","doc":"Sets up an isolated Supervisor with unique naming and automatic cleanup.","ref":"Supertester.OTPHelpers.html#setup_isolated_supervisor/3"},{"type":"function","title":"Parameters - Supertester.OTPHelpers.setup_isolated_supervisor/3","doc":"- `module` - The Supervisor module to start\n- `test_name` - Test context for unique naming (optional)\n- `opts` - Options passed to Supervisor.start_link (optional)","ref":"Supertester.OTPHelpers.html#setup_isolated_supervisor/3-parameters"},{"type":"function","title":"Returns - Supertester.OTPHelpers.setup_isolated_supervisor/3","doc":"`{:ok, supervisor_pid}` or `{:error, reason}`","ref":"Supertester.OTPHelpers.html#setup_isolated_supervisor/3-returns"},{"type":"function","title":"Example - Supertester.OTPHelpers.setup_isolated_supervisor/3","doc":"{:ok, supervisor} = setup_isolated_supervisor(MySupervisor, \"my_test\")","ref":"Supertester.OTPHelpers.html#setup_isolated_supervisor/3-example"},{"type":"function","title":"Supertester.OTPHelpers.wait_for_genserver_sync/2","doc":"Waits for a GenServer to be synchronized and responsive.\n\nUses GenServer.call with a sync message instead of Process.sleep.","ref":"Supertester.OTPHelpers.html#wait_for_genserver_sync/2"},{"type":"function","title":"Parameters - Supertester.OTPHelpers.wait_for_genserver_sync/2","doc":"- `server` - The GenServer pid or name\n- `timeout` - Timeout in milliseconds (default: 1000)","ref":"Supertester.OTPHelpers.html#wait_for_genserver_sync/2-parameters"},{"type":"function","title":"Returns - Supertester.OTPHelpers.wait_for_genserver_sync/2","doc":"`:ok` if server is responsive, `{:error, reason}` otherwise","ref":"Supertester.OTPHelpers.html#wait_for_genserver_sync/2-returns"},{"type":"function","title":"Example - Supertester.OTPHelpers.wait_for_genserver_sync/2","doc":"wait_for_genserver_sync(server, 5000)","ref":"Supertester.OTPHelpers.html#wait_for_genserver_sync/2-example"},{"type":"function","title":"Supertester.OTPHelpers.wait_for_process_death/2","doc":"Waits for a process to terminate.","ref":"Supertester.OTPHelpers.html#wait_for_process_death/2"},{"type":"function","title":"Parameters - Supertester.OTPHelpers.wait_for_process_death/2","doc":"- `pid` - The process to wait for\n- `timeout` - Timeout in milliseconds (default: 1000)","ref":"Supertester.OTPHelpers.html#wait_for_process_death/2-parameters"},{"type":"function","title":"Returns - Supertester.OTPHelpers.wait_for_process_death/2","doc":"`{:ok, reason}` if process died, `{:error, :timeout}` if timeout","ref":"Supertester.OTPHelpers.html#wait_for_process_death/2-returns"},{"type":"function","title":"Example - Supertester.OTPHelpers.wait_for_process_death/2","doc":"ref = Process.monitor(pid)\n    Process.exit(pid, :kill)\n    {:ok, :killed} = wait_for_process_death(pid)","ref":"Supertester.OTPHelpers.html#wait_for_process_death/2-example"},{"type":"function","title":"Supertester.OTPHelpers.wait_for_process_restart/3","doc":"Waits for a process to restart after termination.","ref":"Supertester.OTPHelpers.html#wait_for_process_restart/3"},{"type":"function","title":"Parameters - Supertester.OTPHelpers.wait_for_process_restart/3","doc":"- `process_name` - The registered name of the process\n- `original_pid` - The PID before restart\n- `timeout` - Timeout in milliseconds (default: 1000)","ref":"Supertester.OTPHelpers.html#wait_for_process_restart/3-parameters"},{"type":"function","title":"Returns - Supertester.OTPHelpers.wait_for_process_restart/3","doc":"`{:ok, new_pid}` if process restarted, `{:error, reason}` otherwise","ref":"Supertester.OTPHelpers.html#wait_for_process_restart/3-returns"},{"type":"function","title":"Example - Supertester.OTPHelpers.wait_for_process_restart/3","doc":"original_pid = GenServer.whereis(MyServer)\n    GenServer.stop(MyServer)\n    {:ok, new_pid} = wait_for_process_restart(MyServer, original_pid)","ref":"Supertester.OTPHelpers.html#wait_for_process_restart/3-example"},{"type":"function","title":"Supertester.OTPHelpers.wait_for_supervisor_restart/2","doc":"Waits for a supervisor to finish starting all its children.","ref":"Supertester.OTPHelpers.html#wait_for_supervisor_restart/2"},{"type":"function","title":"Parameters - Supertester.OTPHelpers.wait_for_supervisor_restart/2","doc":"- `supervisor` - The supervisor pid or name\n- `timeout` - Timeout in milliseconds (default: 1000)","ref":"Supertester.OTPHelpers.html#wait_for_supervisor_restart/2-parameters"},{"type":"function","title":"Returns - Supertester.OTPHelpers.wait_for_supervisor_restart/2","doc":"`:ok` if supervisor is ready, `{:error, reason}` otherwise","ref":"Supertester.OTPHelpers.html#wait_for_supervisor_restart/2-returns"},{"type":"function","title":"Example - Supertester.OTPHelpers.wait_for_supervisor_restart/2","doc":"{:ok, supervisor} = setup_isolated_supervisor(MySupervisor)\n    wait_for_supervisor_restart(supervisor)","ref":"Supertester.OTPHelpers.html#wait_for_supervisor_restart/2-example"},{"type":"module","title":"Supertester.Assertions","doc":"Custom assertions for OTP patterns and system behavior.\n\nThis module provides specialized assertions that understand OTP patterns,\nmaking tests more expressive and providing better error messages.","ref":"Supertester.Assertions.html"},{"type":"module","title":"Usage - Supertester.Assertions","doc":"import Supertester.Assertions\n\n    test \"my test\" do\n      assert_process_alive(server_pid)\n      assert_genserver_responsive(server_pid)\n    end","ref":"Supertester.Assertions.html#module-usage"},{"type":"function","title":"Supertester.Assertions.assert_all_children_alive/1","doc":"Asserts that all children of a supervisor are alive.","ref":"Supertester.Assertions.html#assert_all_children_alive/1"},{"type":"function","title":"Parameters - Supertester.Assertions.assert_all_children_alive/1","doc":"- `supervisor` - The supervisor pid or name","ref":"Supertester.Assertions.html#assert_all_children_alive/1-parameters"},{"type":"function","title":"Example - Supertester.Assertions.assert_all_children_alive/1","doc":"assert_all_children_alive(supervisor)","ref":"Supertester.Assertions.html#assert_all_children_alive/1-example"},{"type":"function","title":"Supertester.Assertions.assert_child_count/2","doc":"Asserts that a supervisor has the expected number of children.","ref":"Supertester.Assertions.html#assert_child_count/2"},{"type":"function","title":"Parameters - Supertester.Assertions.assert_child_count/2","doc":"- `supervisor` - The supervisor pid or name\n- `expected_count` - The expected number of children","ref":"Supertester.Assertions.html#assert_child_count/2-parameters"},{"type":"function","title":"Example - Supertester.Assertions.assert_child_count/2","doc":"assert_child_count(supervisor, 3)","ref":"Supertester.Assertions.html#assert_child_count/2-example"},{"type":"function","title":"Supertester.Assertions.assert_genserver_handles_message/3","doc":"Asserts that a GenServer handles a message correctly.","ref":"Supertester.Assertions.html#assert_genserver_handles_message/3"},{"type":"function","title":"Parameters - Supertester.Assertions.assert_genserver_handles_message/3","doc":"- `server` - The GenServer pid or name\n- `message` - The message to send\n- `expected_response` - The expected response","ref":"Supertester.Assertions.html#assert_genserver_handles_message/3-parameters"},{"type":"function","title":"Example - Supertester.Assertions.assert_genserver_handles_message/3","doc":"assert_genserver_handles_message(server, :get_counter, {:ok, 5})","ref":"Supertester.Assertions.html#assert_genserver_handles_message/3-example"},{"type":"function","title":"Supertester.Assertions.assert_genserver_responsive/1","doc":"Asserts that a GenServer is responsive.","ref":"Supertester.Assertions.html#assert_genserver_responsive/1"},{"type":"function","title":"Parameters - Supertester.Assertions.assert_genserver_responsive/1","doc":"- `server` - The GenServer pid or name","ref":"Supertester.Assertions.html#assert_genserver_responsive/1-parameters"},{"type":"function","title":"Example - Supertester.Assertions.assert_genserver_responsive/1","doc":"assert_genserver_responsive(server)","ref":"Supertester.Assertions.html#assert_genserver_responsive/1-example"},{"type":"function","title":"Supertester.Assertions.assert_genserver_state/2","doc":"Asserts that a GenServer has the expected state.","ref":"Supertester.Assertions.html#assert_genserver_state/2"},{"type":"function","title":"Parameters - Supertester.Assertions.assert_genserver_state/2","doc":"- `server` - The GenServer pid or name\n- `expected_state` - The expected state or a function to test the state","ref":"Supertester.Assertions.html#assert_genserver_state/2-parameters"},{"type":"function","title":"Example - Supertester.Assertions.assert_genserver_state/2","doc":"assert_genserver_state(server, %{counter: 5})\n    assert_genserver_state(server, fn state -> state.counter > 0 end)","ref":"Supertester.Assertions.html#assert_genserver_state/2-example"},{"type":"function","title":"Supertester.Assertions.assert_memory_usage_stable/2","doc":"Asserts that memory usage remains stable during an operation.","ref":"Supertester.Assertions.html#assert_memory_usage_stable/2"},{"type":"function","title":"Parameters - Supertester.Assertions.assert_memory_usage_stable/2","doc":"- `operation_fun` - Function to execute\n- `tolerance` - Memory tolerance as a percentage (default: 0.1)","ref":"Supertester.Assertions.html#assert_memory_usage_stable/2-parameters"},{"type":"function","title":"Example - Supertester.Assertions.assert_memory_usage_stable/2","doc":"assert_memory_usage_stable(fn -> \n      Enum.each(1..1000, fn _ -> GenServer.call(server, :increment) end)\n    end, 0.05)","ref":"Supertester.Assertions.html#assert_memory_usage_stable/2-example"},{"type":"function","title":"Supertester.Assertions.assert_no_process_leaks/1","doc":"Asserts that no processes are leaked during an operation.","ref":"Supertester.Assertions.html#assert_no_process_leaks/1"},{"type":"function","title":"Parameters - Supertester.Assertions.assert_no_process_leaks/1","doc":"- `operation_fun` - Function to execute","ref":"Supertester.Assertions.html#assert_no_process_leaks/1-parameters"},{"type":"function","title":"Example - Supertester.Assertions.assert_no_process_leaks/1","doc":"assert_no_process_leaks(fn ->\n      {:ok, server} = GenServer.start_link(MyServer, [])\n      GenServer.stop(server)\n    end)","ref":"Supertester.Assertions.html#assert_no_process_leaks/1-example"},{"type":"function","title":"Supertester.Assertions.assert_performance_within_bounds/2","doc":"Asserts that performance is within expected bounds.","ref":"Supertester.Assertions.html#assert_performance_within_bounds/2"},{"type":"function","title":"Parameters - Supertester.Assertions.assert_performance_within_bounds/2","doc":"- `benchmark_result` - Result from performance testing\n- `expectations` - Map of performance expectations","ref":"Supertester.Assertions.html#assert_performance_within_bounds/2-parameters"},{"type":"function","title":"Example - Supertester.Assertions.assert_performance_within_bounds/2","doc":"expectations = %{max_time: 1000, max_memory: 1_000_000}\n    assert_performance_within_bounds(result, expectations)","ref":"Supertester.Assertions.html#assert_performance_within_bounds/2-example"},{"type":"function","title":"Supertester.Assertions.assert_process_alive/1","doc":"Asserts that a process is alive.","ref":"Supertester.Assertions.html#assert_process_alive/1"},{"type":"function","title":"Parameters - Supertester.Assertions.assert_process_alive/1","doc":"- `pid` - The process ID to check","ref":"Supertester.Assertions.html#assert_process_alive/1-parameters"},{"type":"function","title":"Example - Supertester.Assertions.assert_process_alive/1","doc":"assert_process_alive(server_pid)","ref":"Supertester.Assertions.html#assert_process_alive/1-example"},{"type":"function","title":"Supertester.Assertions.assert_process_dead/1","doc":"Asserts that a process is dead.","ref":"Supertester.Assertions.html#assert_process_dead/1"},{"type":"function","title":"Parameters - Supertester.Assertions.assert_process_dead/1","doc":"- `pid` - The process ID to check","ref":"Supertester.Assertions.html#assert_process_dead/1-parameters"},{"type":"function","title":"Example - Supertester.Assertions.assert_process_dead/1","doc":"assert_process_dead(old_pid)","ref":"Supertester.Assertions.html#assert_process_dead/1-example"},{"type":"function","title":"Supertester.Assertions.assert_process_restarted/2","doc":"Asserts that a process has been restarted.","ref":"Supertester.Assertions.html#assert_process_restarted/2"},{"type":"function","title":"Parameters - Supertester.Assertions.assert_process_restarted/2","doc":"- `process_name` - The registered name of the process\n- `original_pid` - The original PID before restart","ref":"Supertester.Assertions.html#assert_process_restarted/2-parameters"},{"type":"function","title":"Example - Supertester.Assertions.assert_process_restarted/2","doc":"original_pid = GenServer.whereis(MyServer)\n    GenServer.stop(MyServer)\n    assert_process_restarted(MyServer, original_pid)","ref":"Supertester.Assertions.html#assert_process_restarted/2-example"},{"type":"function","title":"Supertester.Assertions.assert_supervisor_strategy/2","doc":"Asserts that a supervisor has the expected strategy.","ref":"Supertester.Assertions.html#assert_supervisor_strategy/2"},{"type":"function","title":"Parameters - Supertester.Assertions.assert_supervisor_strategy/2","doc":"- `supervisor` - The supervisor pid or name\n- `expected_strategy` - The expected supervision strategy","ref":"Supertester.Assertions.html#assert_supervisor_strategy/2-parameters"},{"type":"function","title":"Example - Supertester.Assertions.assert_supervisor_strategy/2","doc":"assert_supervisor_strategy(supervisor, :one_for_one)","ref":"Supertester.Assertions.html#assert_supervisor_strategy/2-example"},{"type":"extras","title":"Supertester","doc":"# Supertester\n\n \n   \n \n\n \n     \n     \n     \n \n\n**A battle-hardened testing toolkit for building robust and resilient Elixir & OTP applications.**\n\n---","ref":"readme.html"},{"type":"extras","title":"The Problem: Flaky OTP Tests - Supertester","doc":"Are you tired of...\n- 😫 **Flaky tests** that fail randomly due to race conditions?\n- 📛 **`GenServer` name clashes** when running tests with `async: true`?\n- 🕰️ Littering your test suite with `Process.sleep/1` and hoping for the best?\n- 🤷‍♂️ Struggling to test process crashes, restarts, and complex supervision trees?\n\nWriting tests for concurrent systems is hard. Traditional testing methods often lead to fragile, non-deterministic, and slow test suites.","ref":"readme.html#the-problem-flaky-otp-tests"},{"type":"extras","title":"The Solution: Supertester - Supertester","doc":"`Supertester` provides a comprehensive suite of tools to write clean, deterministic, and reliable tests for your OTP applications. It replaces fragile timing hacks with robust synchronization patterns and provides powerful helpers for simulating and asserting complex OTP behaviors.\n\nWith `Supertester`, you can build a test suite that is **fast**, **parallel**, and **trustworthy**.","ref":"readme.html#the-solution-supertester"},{"type":"extras","title":"Key Features - Supertester","doc":"- ✅ **Rock-Solid Test Isolation**: Run all your tests with `async: true` without fear of process name collisions or state leakage.\n- 🔄 **Deterministic Synchronization**: No more `Process.sleep/1`. Use helpers that wait for processes to be ready, `cast`s to be processed, and supervisors to stabilize.\n- 🤖 **Powerful OTP Assertions**: Go beyond `assert`. Use `assert_process_restarted/2`, `assert_genserver_state/2`, and `assert_all_children_alive/1` for more expressive tests.\n- ✨ **Effortless Setup & Teardown**: Start isolated `GenServer`s and `Supervisor`s with a single line and trust `Supertester` to handle all the cleanup.\n- 💥 **Resilience & Chaos Testing**: Tools for testing crash recovery, supervision strategies, and system stability under stress.","ref":"readme.html#key-features"},{"type":"extras","title":"Installation - Supertester","doc":"Add `supertester` as a dependency in your `mix.exs` file. It's only needed for the `:test` environment.\n\n```elixir\ndef deps do\n  [\n    {:supertester, \"~> 0.1.0\", only: :test}\n  ]\nend\n```\n\nThen, run `mix deps.get` to install.","ref":"readme.html#installation"},{"type":"extras","title":"Quick Start: From Flaky to Robust - Supertester","doc":"See how `Supertester` transforms a common, fragile test pattern into a robust, deterministic one.\n\n#### Before: The Flaky Way\n\n```elixir\n# test/my_app/counter_test.exs\ndefmodule MyApp.CounterTest do\n  use ExUnit.Case, async: false # <-- Forced to run sequentially\n\n  test \"incrementing the counter\" do\n    # Manual setup, prone to name conflicts\n    {:ok, _pid} = start_supervised({Counter, name: Counter})\n\n    GenServer.cast(Counter, :increment)\n    Process.sleep(50) # <-- Fragile, timing-dependent guess\n\n    state = GenServer.call(Counter, :state)\n    assert state.count == 1\n  end\nend\n```\n\n#### After: The Supertester Way\n\n```elixir\n# test/my_app/counter_test.exs\ndefmodule MyApp.CounterTest do\n  use ExUnit.Case, async: true # <-- Fully parallel!\n\n  # Import the tools you need\n  import Supertester.OTPHelpers\n  import Supertester.GenServerHelpers\n  import Supertester.Assertions\n\n  test \"incrementing the counter\" do\n    # Isolated setup with automatic cleanup, no name clashes\n    {:ok, counter_pid} = setup_isolated_genserver(Counter)\n\n    # Deterministic sync: ensures the cast is processed before continuing\n    :ok = cast_and_sync(counter_pid, :increment)\n\n    # Expressive, OTP-aware assertion for checking state\n    assert_genserver_state(counter_pid, fn state -> state.count == 1 end)\n  end\nend\n```","ref":"readme.html#quick-start-from-flaky-to-robust"},{"type":"extras","title":"Core API Highlights - Supertester","doc":"`Supertester` is organized into several modules, each targeting a specific area of OTP testing.\n\n### `Supertester.OTPHelpers`\nFor setting up and managing isolated OTP processes.\n- `setup_isolated_genserver/3`: Starts a `GenServer` with a unique name and automatic cleanup.\n- `setup_isolated_supervisor/3`: Starts a `Supervisor` with a unique name and automatic cleanup.\n- `wait_for_process_restart/3`: Blocks until a supervised process has been terminated and restarted.\n- `wait_for_genserver_sync/2`: Ensures a `GenServer` is alive and responsive.\n\n### `Supertester.GenServerHelpers`\nFor interacting with and testing `GenServer`s.\n- `cast_and_sync/3`: Sends a `cast` and waits for a follow-up `call` to confirm it was processed.\n- `get_server_state_safely/1`: Fetches `GenServer` state without crashing if the process is down.\n- `test_server_crash_recovery/2`: Simulates a process crash and verifies its recovery by the supervisor.\n- `concurrent_calls/3`: Stress-tests a `GenServer` with many concurrent requests.\n\n### `Supertester.Assertions`\nCustom, OTP-aware assertions for more meaningful tests.\n- `assert_process_alive/1` & `assert_process_dead/1`\n- `assert_genserver_state/2`: Asserts the `GenServer`'s internal state matches a value or passes a function check.\n- `assert_child_count/2`: Asserts a supervisor has an exact number of active children.\n- `assert_all_children_alive/1`: Checks that all children in a supervision tree are running.\n- `assert_no_process_leaks/1`: Ensures an operation cleans up all the processes it spawns.\n\n### `Supertester.UnifiedTestFoundation`\nProvides advanced, case-level isolation for complex scenarios.\n\n```elixir\ndefmodule MyApp.MyAdvancedTest do\n  use ExUnit.Case\n  # Choose an isolation level for the entire test module.\n  # :full_isolation provides sandboxed processes and ETS tables.\n  use Supertester.UnifiedTestFoundation, isolation: :full_isolation\n\n  test \"this test runs in a complete sandbox\", context do\n    # `context.isolation_context` holds info about the sandbox.\n    # All processes started via Supertester helpers are tracked and auto-cleaned.\n    {:ok, server} = Supertester.OTPHelpers.setup_isolated_genserver(MyServer)\n    # ... your isolated test logic ...\n  end\nend\n```","ref":"readme.html#core-api-highlights"},{"type":"extras","title":"Contributing - Supertester","doc":"Contributions are welcome! If you'd like to help improve `Supertester`, please feel free to:\n1.  Fork the repository.\n2.  Create a new feature branch.\n3.  Add your feature or bug fix.\n4.  Ensure all new code is covered by tests.\n5.  Open a pull request.","ref":"readme.html#contributing"},{"type":"extras","title":"License - Supertester","doc":"This project is licensed under the MIT License.","ref":"readme.html#license"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [0.1.0] - 2025-07-22","ref":"changelog.html"},{"type":"extras","title":"Added - Changelog","doc":"- Initial release of Supertester\n- Multi-repository test orchestration and execution framework\n- OTP-compliant testing utilities to replace `Process.sleep/1`\n- Test isolation support enabling `async: true`\n- GenServer testing helpers with proper synchronization\n- Supervisor testing utilities for supervision tree validation\n- Performance testing framework for benchmarking and load testing\n- Chaos engineering helpers for resilience testing\n- Custom OTP-aware assertions\n- Unified test foundation with multiple isolation modes\n- Complete elimination of GenServer registration conflicts\n- Zero test failure guarantee across monorepo structures","ref":"changelog.html#added"},{"type":"extras","title":"Features - Changelog","doc":"- `Supertester.UnifiedTestFoundation` - Test isolation and foundation patterns\n- `Supertester.OTPHelpers` - OTP-compliant testing utilities\n- `Supertester.GenServerHelpers` - GenServer-specific test patterns\n- `Supertester.Assertions` - Custom OTP-aware assertions\n\n[0.1.0]: https://github.com/nshkrdotcom/supertester/releases/tag/v0.1.0","ref":"changelog.html#features"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}